#include <conio.h>
#include <stdio.h>
#include <windows.h>

#ifndef _MAX_PATH
# define _MAX_PATH   260
#endif

// -------------------------------------

void usage ()
{
    printf (
            "Deletes a directory and all the subdirectories and files in it.\n"
            "\n"
            "To delete one or more files and directories:\n"
            "DELTREE [/Y] [drive:]path [[drive:]path[...]]\n"
            "\n"
            "  /Y              Suppresses prompting to confirm you want to delete\n"
            "                  the subdirectory.\n"
            "  [drive:]path    Specifies the name of the directory you want to delete.\n"
           );
}

void fullname ( char *buffer, const char *path, const char *name )
{
    char *p = buffer;
    p += sprintf ( p, "%s", path );
    if ( p > buffer && *(p-1) != '\\' ) p += sprintf ( p, "\\" );
    sprintf ( p, "%s", name );
}

int wipe_dir ( char *path )
{
    WIN32_FIND_DATA FindFileData;
    HANDLE          hFind;
    char            temp[_MAX_PATH*2];
    const char      *fn;
    int             errors = 0;

    if ( path[0] && (path[strlen (path)-1] == '\\' || path[strlen (path)-1] == '/') )
        path[strlen (path)-1] = '\0';

    sprintf ( temp, "%s\\*", path );

    hFind = FindFirstFile ( temp, &FindFileData );

    if ( hFind != INVALID_HANDLE_VALUE ) {
        fn = FindFileData.cAlternateFileName;
        if ( !fn || fn[0] == '\0' ) fn = FindFileData.cFileName;
        fullname ( temp, path, fn );

        if ( FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {   // directory
            if ( lstrcmp (FindFileData.cFileName, ".") && lstrcmp (FindFileData.cFileName, "..") ) {
                errors += wipe_dir ( temp );
            }
        } else {                                                            // file
            if ( !DeleteFile (temp) ) errors++;
        }

        while ( FindNextFile (hFind, &FindFileData) ) {
            fn = FindFileData.cAlternateFileName;
            if ( !fn || fn[0] == '\0' ) fn = FindFileData.cFileName;
            fullname ( temp, path, fn );

            if ( FindFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY ) {   // directory
                if ( lstrcmp (FindFileData.cFileName, ".") && lstrcmp (FindFileData.cFileName, "..") ) {
                    errors += wipe_dir ( temp );
                }
            } else {                                                            // file
                if ( !DeleteFile (temp) ) errors++;
            }
        }

        FindClose ( hFind );
    }

    if ( !RemoveDirectory (path) ) errors++;

    return errors;
}

int isdir ( const char *path )
{
    DWORD attr = GetFileAttributes ( path );
    if ( attr == -1 ) return 0;
    return attr & FILE_ATTRIBUTE_DIRECTORY;
}

int main ( int argc, char **argv )
{
    int i, errors = 0;
    int noprompt = 0;

    if ( argc < 2 ) {
        usage ();
        return 0;
    }

    for ( i = 1; i < argc; i++ ) {
        if ( (argv[i][0] == '-' || argv[i][0] == '/') &&
             (argv[i][1] == 'y' || argv[i][1] == 'Y') ) {
            noprompt = 1;
            continue;
        }

        if ( isdir (argv[i]) ) {
            if ( !noprompt ) {
                int dela = -1;   // -1: empty, 0: no, 1: yes

                while (1) {
                    int ch;
                    printf ( "Delete directory \"%s\" and all its subdirectories? [yn] ", argv[i] );
                    while ( !_kbhit () ) Sleep (10);
                    ch = _getch ();

                    if ( ch == 'y' || ch == 'Y' ) dela = 1;
                    if ( ch == 'n' || ch == 'N' ) dela = 0;

                    switch ( dela ) {
                    case 0:
                        printf ( "n\r" ); break;
                    case 1:
                        printf ( "y\r" ); break;
                    case -1:
                    default:
                        break;
                    }

                    if ( dela >= 0 && ch == '\r' || ch == '\n' ) break;
                }

                printf ( "\n" );

                if ( !dela ) continue;
            }

            printf ( "Deleting %s...\n", argv[i] );
            errors += wipe_dir ( argv[i] );
        } else {
            printf ( "Deleting %s...\n", argv[i] );
            if ( !DeleteFile (argv[i]) ) errors++;
        }
    }

    return errors != 0;
}
